@@ -1,30 +1,37 @@
-/*
-*  C Implementation: fileops.c
-*
-* (Note: read_config_file() was made possible by studying the file prefs.c in gtkpod:
-*  URL: http://www.gtkpod.org/
-*  URL: http://gtkpod.sourceforge.net/
-*  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>.
-*  Licensed under GNU GPL v2.
-*  This code is a nearly complete rewrite but I would like to express my thanks.)
-*
-*
-* Description: File operations - together, fileops.h and fileops.c contain 
-* all code involving disk operations.  The intention is to make it easier to
-* port tuxmath to other operating systems, as code to read and write as 
-* well as paths and file locations may be more OS-dependent.
-*
-* This file contains functions to read and write config files.
-* The config file contains name-value pairs, one pair per line, to control
-* settings for the behavior of Tuxmath.
-*
-* Code for loading program data from disk is now also found here.
-* 
-* Author: David Bruce <dbruce@tampabay.rr.com>, (C) 2006
-*
-* Copyright: See COPYING file that comes with this distribution (briefly, GNU GPL)
-*
-*/
+/* fileops.c
+  
+   All code involving disk operations is intended to be located here.
+
+  (Note: read_config_file() was made possible by studying the file prefs.c in gtkpod:
+   URL: http://www.gtkpod.org/
+   URL: http://gtkpod.sourceforge.net/
+   Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>.
+   Licensed under GNU GPL v2+.
+   This code is a nearly complete rewrite but I would like to express my thanks.)
+  
+  Copyright 2006, 2007, 2008, 2009, 2010.
+  Author: David Bruce, Tim Holy, Boleslaw Kulbabinski, Brendan Luchen.
+  Project email: <tuxmath-devel@lists.sourceforge.net>
+  Project website: http://tux4kids.alioth.debian.org
+
+
+fileops.c is part of "Tux, of Math Command", a.k.a. "tuxmath".
+
+Tuxmath is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+Tuxmath is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+
+
+
 
 #include <stdio.h>
 
@@ -294,9 +301,7 @@ int read_user_config_file(void)
   get_user_data_dir_with_subdir(opt_path);
   strcat(opt_path, OPTIONS_FILENAME);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_user_config_file() full path to config file is: = %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_user_config_file() full path to config file is: = %s\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
@@ -331,10 +336,7 @@ int read_named_config_file(const char* fn)
     free(last_config_file_name);
   last_config_file_name = strdup(filename);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_named_config_file() filename is: = %s\n", filename);
-  #endif
-
+  DEBUGMSG(debug_fileops, "In read_named_config_file() filename is: = %s\n", filename);
 
   /* First look in current working directory:  */
   getcwd(opt_path, PATH_MAX); /* get current working directory */
@@ -345,18 +347,12 @@ int read_named_config_file(const char* fn)
   }
   strcat(opt_path, filename); /* tack on filename              */
 
-
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_named_config_file() checking for %s (cwd)\n", opt_path);
-  #endif
-
+  DEBUGMSG(debug_fileops, "In read_named_config_file() checking for %s (cwd)\n", opt_path);
 
   fp = fopen(opt_path, "r");  /* try to open file */
   if (fp) /* file exists */
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nFound %s\n", opt_path);
-    #endif
+    DEBUGMSG(debug_fileops, "Found %s\n", opt_path);
 
     if (read_config_file(fp, USER_CONFIG_FILE))
     {
@@ -384,16 +380,12 @@ int read_named_config_file(const char* fn)
     strcat(opt_path, filename);
   }
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_named_config_file() checking for %s (abs)\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_named_config_file() checking for %s (abs)\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nFound %s\n", opt_path);
-    #endif
+    DEBUGMSG(debug_fileops, "Found %s\n", opt_path);
 
     if (read_config_file(fp, USER_CONFIG_FILE))
     {
@@ -414,16 +406,12 @@ int read_named_config_file(const char* fn)
   strcat(opt_path, "/missions/");
   strcat(opt_path, filename);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_named_config_file() checking for %s (missions)\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_named_config_file() checking for %s (missions)\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nFound %s\n", opt_path);
-    #endif
+    DEBUGMSG(debug_fileops, "Found %s\n", opt_path);
 
     if (read_config_file(fp, USER_CONFIG_FILE))
     {
@@ -443,16 +431,12 @@ int read_named_config_file(const char* fn)
   strcat(opt_path, "/missions/lessons/");
   strcat(opt_path, filename);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_named_config_file() checking for %s (missions/lessons)\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_named_config_file() checking for %s (missions/lessons)\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nFound %s\n", opt_path);
-    #endif
+    DEBUGMSG(debug_fileops, "Found %s\n", opt_path);
 
     if (read_config_file(fp, USER_CONFIG_FILE))
     {
@@ -472,16 +456,12 @@ int read_named_config_file(const char* fn)
   strcat(opt_path, "/missions/arcade/");
   strcat(opt_path, filename);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_named_config_file() checking for %s (missions/arcade)\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_named_config_file() checking for %s (missions/arcade)\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nFound %s\n", opt_path);
-    #endif
+    DEBUGMSG(debug_fileops, "Found %s\n", opt_path);
 
     if (read_config_file(fp, USER_CONFIG_FILE))
     {
@@ -501,16 +481,12 @@ int read_named_config_file(const char* fn)
   get_user_data_dir_with_subdir(opt_path);
   strcat(opt_path, filename);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_named_config_file() checking for %s (.tuxmath)\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_named_config_file() checking for %s (.tuxmath)\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nFound %s\n", opt_path);
-    #endif
+    DEBUGMSG(debug_fileops, "Found %s\n", opt_path);
 
     if (read_config_file(fp, USER_CONFIG_FILE))
     {
@@ -532,16 +508,12 @@ int read_named_config_file(const char* fn)
   strcat(opt_path, "/");
   strcat(opt_path, filename);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_named_config_file() checking for %s (home)\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_named_config_file() checking for %s (home)\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nFound %s\n", opt_path);
-    #endif
+    DEBUGMSG(debug_fileops, "Found %s\n", opt_path);
 
     if (read_config_file(fp, USER_CONFIG_FILE))
     {
@@ -557,9 +529,7 @@ int read_named_config_file(const char* fn)
   }
 
   /* Could not find file (or read it if found) in any location: */
-  #ifdef TUXMATH_DEBUG
-  printf("\nread_named_config_file() could not find/read: %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "read_named_config_file() could not find/read: %s\n", opt_path);
   return 0;
 }
 
@@ -596,17 +566,12 @@ int parse_lesson_file_directory(void)
     return 0;
   }
 
-#ifdef TUXMATH_DEBUG
-  fprintf(stderr, "lesson_path is: %s\n", lesson_path);
-#endif
+  DEBUGMSG(debug_fileops, "lesson_path is: %s\n", lesson_path);
 
   /* Believe we now have complete scandir() for all platforms :) */
   num_lessons = scandir(lesson_path, &lesson_list_dirents, is_lesson_file, alphasort);
 
-
-#ifdef TUXMATH_DEBUG
-  fprintf(stderr, "num_lessons is: %d\n", num_lessons);
-#endif
+  DEBUGMSG(debug_fileops, "num_lessons is: %d\n", num_lessons);
 
   if (num_lessons < 0) {
     perror("scanning lesson directory");
@@ -642,9 +607,7 @@ int parse_lesson_file_directory(void)
     if (nchars < 0 || nchars >= NAME_BUF_SIZE)
       continue;
 
-#ifdef TUXMATH_DEBUG
-    fprintf(stderr, "Found lesson file %d:\t%s\n", lessons, lesson_list_filenames[lessons]);
-#endif
+    DEBUGMSG(debug_fileops, "Found lesson file %d:\t%s\n", lessons, lesson_list_filenames[lessons]);
 
     /* load the name for the lesson from the file ... (1st line) */
     tempFile = fopen(lesson_list_filenames[lessons], "r");
@@ -740,9 +703,7 @@ int read_goldstars(void)
   get_user_data_dir_with_subdir(opt_path);
   strcat(opt_path, GOLDSTAR_FILENAME);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_goldstars() full path to file is: = %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_goldstars() full path to file is: = %s\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
@@ -776,9 +737,7 @@ int write_goldstars(void)
   get_user_data_dir_with_subdir(opt_path);
   strcat(opt_path, GOLDSTAR_FILENAME);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn write_goldstars() full path to file is: = %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In write_goldstars() full path to file is: = %s\n", opt_path);
 
   fp = fopen(opt_path, "w");
   if (fp)
@@ -808,9 +767,7 @@ int high_scores_found_in_user_dir(void)
   get_user_data_dir_with_subdir(opt_path);
   strcat(opt_path, HIGHSCORE_FILENAME);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_high_scores() full path to file is: = %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_high_scores() full path to file is: = %s\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
@@ -856,9 +813,7 @@ int read_high_scores(void)
     strncpy(opt_path,high_scores_file_path,PATH_MAX);
   strcat(opt_path, HIGHSCORE_FILENAME);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn read_high_scores() full path to file is: = %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In read_high_scores() full path to file is: = %s\n", opt_path);
 
   fp = fopen(opt_path, "r");
   if (fp) /* file exists */
@@ -933,9 +888,7 @@ int append_high_score(int tableid,int score,char *player_name)
     strncpy(opt_path,high_scores_file_path,PATH_MAX);
   strcat(opt_path, HIGHSCORE_FILENAME);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn write_high_scores() full path to file is: = %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In write_high_scores() full path to file is: = %s\n", opt_path);
 
   fp = fopen(opt_path, "a");
   if (fp)
@@ -1062,17 +1015,13 @@ int read_config_file(FILE *fp, int file_type)
   char buf[PATH_MAX];
   char *parameter, *param_begin, *param_end, *value, *value_end;
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nEntering read_config_file()\n");
-  #endif
+  DEBUGMSG(debug_fileops, "Entering read_config_file()\n");
 
   /* get out if file pointer invalid: */
   if(!fp)
   {
-    #ifdef TUXMATH_DEBUG
-    printf("config file pointer invalid!\n");
-    printf("Leaving read_config_file()\n");
-    #endif
+    DEBUGMSG(debug_fileops, "config file pointer invalid!\n");
+    DEBUGMSG(debug_fileops, "Leaving read_config_file()\n");
 
     fprintf(stderr, "config file pointer invalid!\n");
     return 0;
@@ -1081,12 +1030,21 @@ int read_config_file(FILE *fp, int file_type)
   /* make sure we start at beginning: */
   rewind(fp);
 
-  /* read in a line at a time: */
+  /* read in top line (lesson title), removing initial "# "          */ 
+  {
+    char* p;
+    fgets (buf, PATH_MAX, fp);
+    p = buf;
+    while (*p == '#'||isspace(*p))
+      ++p;
+    Opts_SetLessonTitle(p);
+  }
+
+  /* now start over at beginning: */
+  rewind(fp);
+
   while (fgets (buf, PATH_MAX, fp))
   { 
-    #ifdef TUXMATH_DEBUG
-    //printf("Beginning fgets() loop\n");
-    #endif
     /* "parameter" and "value" will contain the non-whitespace chars */
     /* before and after the '=' sign, respectively.  e.g.:           */
     /*                                                               */
@@ -1098,9 +1056,6 @@ int read_config_file(FILE *fp, int file_type)
     /* ignore comment lines */
     if ((buf[0] == ';') || (buf[0] == '#'))
     {
-      #ifdef TUXMATH_DEBUG
-      //printf("Skipping comment line\n");
-      #endif
       continue;
     }
  
@@ -1140,10 +1095,6 @@ int read_config_file(FILE *fp, int file_type)
 
     if (!value || (value == buf))
     {
-      #ifdef TUXMATH_DEBUG
-      //fprintf(stderr, "Error while reading prefs - line with no '='!\n");
-      #endif
-
       free(parameter);
       continue;
     }
@@ -1169,8 +1120,8 @@ int read_config_file(FILE *fp, int file_type)
     /* terminate string here: */
     *value_end = 0;
 
-    tmdprintf("parameter = '%s'\t, length = %zu\n", parameter, strlen(parameter));
-    tmdprintf("value = '%s'\t, length = %zu\t, atoi() = %d\t, atof() = %.2f\n", value, strlen(value), atoi(value), atof(value));
+    DEBUGMSG(debug_fileops, "parameter = '%s'\t, length = %zu\n", parameter, strlen(parameter));
+    DEBUGMSG(debug_fileops, "value = '%s'\t, length = %zu\t, atoi() = %d\t, atof() = %.2f\n", value, strlen(value), atoi(value), atof(value));
     
     /* Now ready to handle each name/value pair! */
     
@@ -1413,12 +1364,11 @@ int read_config_file(FILE *fp, int file_type)
       MC_SetOpt(DIVISION_ALLOWED, 0);
   if (MC_GetOpt(MIN_TYPING_NUM) > MC_GetOpt(MAX_TYPING_NUM) )
       MC_SetOpt(TYPING_PRACTICE_ALLOWED, 0);
-      
-  #ifdef TUXMATH_DEBUG
-  printf("\nAfter file read in:\n");
-  write_config_file(stdout, 0);
-  printf("Leaving read_config_file()\n");
-  #endif
+
+  DEBUGMSG(debug_fileops, "After file read in:\n");
+  DEBUGCODE(debug_fileops)
+    write_config_file(stdout, 0);
+  DEBUGMSG(debug_fileops, "Leaving read_config_file()\n");
 
   return 1;
 }
@@ -1462,9 +1412,7 @@ int write_user_config_file(void)
   get_user_data_dir_with_subdir(opt_path);
   strcat(opt_path, OPTIONS_FILENAME);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn write_user_config_file() full path to config file is: = %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In write_user_config_file() full path to config file is: = %s\n", opt_path);
 
   /* save settings: */
   fp = fopen(opt_path, "w");
@@ -1653,13 +1601,6 @@ int write_config_file(FILE *fp, int verbose)
     "# missed question will reappear. This can be set anywhere  #\n"
     "# from 1 to 10.                                            #\n"
     "#                                                          #\n"
-    "# 'fraction_to_keep' allows a list to be generated that    #\n"
-    "# consists of a randomly-selected subset of the questions  #\n"
-    "# fitting the criteria.  The parameter is a float that     #\n"
-    "# must be greater than 0 and less than or equal to 1. For  #\n"
-    "# example, a value of 0.1 means 10%% of the questions      #\n"
-    "# meeting the criteria will go into the list.              #\n"
-    "#                                                          #\n"
     "# The defaults for these values result in a 'mission'      #\n" 
     "# for Tux that is accomplished by answering all            #\n"
     "# questions correctly with at least one surviving city.    #\n"
@@ -1718,14 +1659,13 @@ int write_config_file(FILE *fp, int verbose)
     "############################################################\n\n";
     
   }
-  
-  tmdprintf("\nEntering write_config_file()\n");
+  DEBUGMSG(debug_fileops, "Entering write_config_file()\n");
 
   /* get out if file pointer null */
   if(!fp)
   {
     fprintf (stderr, "write_config_file() - file pointer invalid/n");
-    tmdprintf("Leaving write_config_file()\n");
+    DEBUGMSG(debug_fileops, "Leaving write_config_file()\n");
     return 0;
   }
   
@@ -1944,9 +1884,7 @@ int write_config_file(FILE *fp, int verbose)
   /* print options pertaining to math questions from MathCards: */
 //  MC_PrintMathOptions(fp, 1);
 
-  #ifdef TUXMATH_DEBUG
-  printf("Leaving write_config_file()\n");
-  #endif
+  DEBUGMSG(debug_fileops, "Leaving write_config_file()\n");
 
   return 1;
 }
@@ -1967,6 +1905,8 @@ int write_pregame_summary(void)
   char filepath1[PATH_MAX];
   char filepath2[PATH_MAX];
 
+  DEBUGMSG(debug_fileops,"Entering write_pregame_summary.\n")
+
   /* Make sure tuxmath dir exists or can be created: */
   if (!find_tuxmath_dir())
   {
@@ -1986,9 +1926,7 @@ int write_pregame_summary(void)
   fp = fopen(filepath1, "r");
   if (fp)
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nIn write_pregame_summary() - removing oldest summary file\n");
-    #endif
+    DEBUGMSG(debug_fileops,"\nIn write_pregame_summary() - removing oldest summary file\n")
 
     fclose(fp);
     remove(filepath1);
@@ -2038,10 +1976,12 @@ int write_pregame_summary(void)
     fprintf(fp, "\n\nNumber of Questions: %d", MC_StartingListLength());
 
     fclose(fp);
+    DEBUGMSG(debug_fileops,"Leaving write_pregame_summary.\n")
     return 1;
   }
   else /* Couldn't write file for some reason: */
   {
+    DEBUGMSG(debug_fileops,"Can't write_pregame_summary.\n")
     return 0;
   }
 }
@@ -2168,17 +2108,13 @@ static int find_tuxmath_dir(void)
   /* find $HOME */
   get_user_data_dir_with_subdir(opt_path);
 
-  #ifdef TUXMATH_DEBUG
-  printf("\nIn find_tuxmath_dir() tuxmath dir is: = %s\n", opt_path);
-  #endif
+  DEBUGMSG(debug_fileops, "In find_tuxmath_dir() tuxmath dir is: = %s\n", opt_path);
 
   /* find out if directory exists - if not, create it: */
   dir_ptr = opendir(opt_path);
   if (dir_ptr)  /* don't leave DIR* open if it was already there */
   {
-    #ifdef TUXMATH_DEBUG
-    printf("\nIn find_tuxmath_dir() tuxmath dir opened OK\n");
-    #endif
+    DEBUGMSG(debug_fileops, "In find_tuxmath_dir() tuxmath dir opened OK\n");
 
     closedir(dir_ptr);
     return 1;
@@ -2196,17 +2132,13 @@ static int find_tuxmath_dir(void)
     fp = fopen(opt_path, "r");
     if (fp)
     {
-      #ifdef TUXMATH_DEBUG
-      printf("\nIn find_tuxmath_dir() - removing old .tuxmath file\n");
-      #endif
+      DEBUGMSG(debug_fileops, "In find_tuxmath_dir() - removing old .tuxmath file\n");
 
       fclose(fp);
       remove(opt_path);
     }
 
-    #ifdef TUXMATH_DEBUG
-    printf("\nIn find_tuxmath_dir() - trying to create .tuxmath dir\n");
-    #endif
+    DEBUGMSG(debug_fileops, "In find_tuxmath_dir() - trying to create .tuxmath dir\n");
 
     //status = mkdir(opt_path, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
 
@@ -2216,9 +2148,7 @@ static int find_tuxmath_dir(void)
     status = mkdir(opt_path);
 #endif
 
-    #ifdef TUXMATH_DEBUG
-    printf("\nIn find_tuxmath_dir() - mkdir returned: %d\n", status);
-    #endif
+    DEBUGMSG(debug_fileops, "In find_tuxmath_dir() - mkdir returned: %d\n", status);
 
     /* mkdir () returns 0 if successful */
     if (0 == status)
